use axum::{routing::get, Router};
use http::header;
use sqlx::{Pool, Postgres};
use state::AppState;
use tracing::Level;
use tracing_subscriber::FmtSubscriber;

const ADMIN_TOK: &str = "meow";

mod admin;
mod chat;
mod markup_response;
mod model;
mod state;
mod ws;

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    let subscriber = FmtSubscriber::builder()
        .with_max_level(Level::TRACE)
        .finish();
    tracing::subscriber::set_global_default(subscriber).expect("setting default subscriber failed");

    let state = AppState::init().await?;

    let app = Router::new()
        .route("/", get(|| async { "<h1>gay</h1>" }))
        .route(
            "/js",
            get(|| async {
                (
                    [(header::CONTENT_TYPE, "text/javascript")],
                    include_str!("../../../static/chat.js"),
                )
            }),
        )
        .route(
            "/css",
            get(|| async {
                (
                    [(header::CONTENT_TYPE, "text/css")],
                    include_str!("../../../static/chat.css"),
                )
            }),
        )
        .route("/:path", get(chat::get).post(chat::post))
        .route("/poll/:msg", get(chat::poll))
        .with_state(state.clone())
        .nest("/admin", admin::router(state.clone()));
    let listener = tokio::net::TcpListener::bind("0.0.0.0:8080").await.unwrap();
    axum::serve(listener, app).await?;

    Ok(())
}
